import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.awt.Color;

/**
 * GradientColor v1.0
 *  by Dalvengyr
 *  
 * Description
 * ¯¯¯¯¯¯¯¯¯¯¯
 *   Generate scaled gradient color to be used for
 *  anything you could think of.
 *  
 * API
 * ¯¯¯
 *  Declare a new instance
 *      | public GradientColor(int length, int scale)
 *          length => max length of the result
 *          scale  => in percentage (1-100)
 *          
 *  Set original color
 *      | public void setOrigin(int red, int green, int blue, int alpha)
 *      
 *  Set target color
 *      | public void setTarget(int red, int green, int blue, int alpha)
 *      
 *  Get current /unchanged/ color
 *      | public Color getCurrentColor()
 *      
 *  Get next color
 *      | public Color getNextColor()
 */
public class GradientColor
{
    private int oRed, oGreen, oBlue, oAlpha; // Original color
    private int cRed, cGreen, cBlue, cAlpha; // Current color
    private int tRed, tGreen, tBlue, tAlpha; // Target color
    private int rRed, rGreen, rBlue, rAlpha; // Turn rate
    private int gLength, gCount, gMod;
    private double gScale;
    
    /*
     * Initialize a GradientColor
     */
    public GradientColor(int length, int scale)
    {
        gLength = length;
        gScale  = scale/100.0;
        gMod    = (int)(gLength*gScale);
    }
    
    /*
     * Set original color (0-255)
     */
    public void setOrigin(int red, int green, int blue, int alpha)
    {
        oRed   = red;
        oGreen = green;
        oBlue  = blue;
        oAlpha = alpha;
        
        cRed   = red;
        cGreen = green;
        cBlue  = blue;
        cAlpha = alpha;
    }
    
    /*
     * Set  target color (0-255)
     */
    public void setTarget(int red, int green, int blue, int alpha)
    {
        tRed   = red;
        tGreen = green;
        tBlue  = blue;
        tAlpha = alpha;
        
        rRed   = (int)((((double)tRed-(double)oRed)/((double)gLength+1))/gScale);
        rGreen = (int)((((double)tGreen-(double)oGreen)/((double)gLength+1))/gScale);
        rBlue  = (int)((((double)tBlue-(double)oBlue)/((double)gLength+1))/gScale);
        rAlpha = (int)((((double)tAlpha-(double)oAlpha)/((double)gLength+1))/gScale);
    }
    
    /*
     * Get current color
     */
    public Color getCurrentColor()
    {
        return new Color(cRed, cGreen, cBlue, cAlpha);
    }
    
    /*
     * Get next color as well as calculating the next color
     * afterwards
     */
    public Color getNextColor()
    {
        int x;
        gCount++;
        if (((gCount-1)/gMod)%2 == 0)
            x = 1;
        else
            x = -1;
        
        /*
        //Debugging
        System.out.println("=======================================");
        System.out.println("Count: " + gCount + " / "  + gLength);
        System.out.println("Red: "   + cRed   + " + (" + (rRed*x)   + ") = " + (cRed   + rRed*x));
        System.out.println("Green: " + cGreen + " + (" + (rGreen*x) + ") = " + (cGreen + rGreen*x));
        System.out.println("Blue: "  + cBlue  + " + (" + (rBlue*x)  + ") = " + (cBlue  + rBlue*x));
        System.out.println("Alpha: " + cAlpha + " + (" + (rAlpha*x) + ") = " + (cBlue  + rAlpha*x));
        */
            
        cRed   = cRed + rRed*x;
        cGreen = cGreen + rGreen*x;
        cBlue  = cBlue + rBlue*x;
        cAlpha = cAlpha + rAlpha*x;
        
        // Reset gradation
        if (gCount == gLength)
        {
            gCount = 0;
            cRed   = oRed;
            cGreen = oGreen;
            cBlue  = oBlue;
            cAlpha = oAlpha;
        }
        
        return new Color(cRed, cGreen, cBlue, cAlpha);
    }
}
